package com.imageresizer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.webkit.MimeTypeMap;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.imageresizer.codec.target.CodecCallback;
import com.imageresizer.codec.target.TargetGif;
import com.imageresizer.utility.BitmapUtility;
import com.imageresizer.utility.StaticOptions;

import java.io.File;
import java.io.FileNotFoundException;

public class ImageResizerModule extends ReactContextBaseJavaModule {
  public static String TAG = "ImageResizerModule";
  public static final String NAME = "ImageResizer";

  ImageResizerModule(ReactApplicationContext context) {
    super(context);
  }

  @Override
  @NonNull
  public String getName() {
    return NAME;
  }


  // Example method
  // See https://reactnative.dev/docs/native-modules-android
  @ReactMethod
  public void multiply(double a, double b, Promise promise) {
    /* for testing purpose */
    promise.resolve(a * b + 5000);

  }

  @ReactMethod
  public void ResizeAnimatedImage(String uri, ReadableMap userOptions, Promise promise) {
    StaticOptions options = new StaticOptions(userOptions);

    File file = new File(uri);
    if (!file.exists()) promise.reject("Error:ResizeAnimatedImage", "File does not exist");

    Context context = getReactApplicationContext();

    String extension = MimeTypeMap.getFileExtensionFromUrl(uri);
    if(extension != null){

      // on complete
      CodecCallback cb = new CodecCallback() {
        @Override
        public void onComplete(String filePath) {
          promise.resolve(filePath);
        }

        @Override
        public void onFailed(String filePath) {
          promise.reject("Error in codec",filePath);
        }
      };

      // Call as per Type GIF/WEBP
      if(extension.equals("gif")){
            GlideResizeGif(uri, cb, options);
      }else if(extension.equals("webp")){
            promise.reject("TEST ERROR","AWEBP");
//            GlideResizeAwebp(uri, cb);
      }else{
        promise.reject("Error:ResizeAnimatedImage", "File extentions is not gif or webp");
      }
    }
    


  }

  private void GlideResizeGif(String uri, CodecCallback callback, StaticOptions options) {
    Context context = getReactApplicationContext();


    // 1. init Target type
    TargetGif gifTarget = new TargetGif(context, callback, options);

    // 2. call Glide
    Glide.with(context)
            .asGif()
            .load(uri)
            .centerInside()
            .override(512, 512)
            .skipMemoryCache(true)
            .diskCacheStrategy(DiskCacheStrategy.NONE)
            .listener(new RequestListener<GifDrawable>() {
              @Override
              public boolean onLoadFailed(@Nullable GlideException e, @Nullable Object model, @NonNull Target<GifDrawable> target, boolean isFirstResource) {
                if(e != null){
                  callback.onFailed(e.getMessage());
                }else{
                  callback.onFailed("Unknown error in Glide decoding GIF");
                }
                return false;
              }

              @Override
              public boolean onResourceReady(@NonNull GifDrawable resource, @NonNull Object model, Target<GifDrawable> target, @NonNull DataSource dataSource, boolean isFirstResource) {
                return false;
              }
            })
            .into(gifTarget);
  }

  private void GlideResizeAwebp(String uri, Promise promise) {
  }

  @ReactMethod
  public void ResizeStaticImage(String uri, ReadableMap userOptions, Promise promise) {
    StaticOptions options = new StaticOptions(userOptions);

    File file = new File(uri);
    if (!file.exists()) {
      promise.reject("File does not exist");
    }

    // create bitmap
    Bitmap bitmap = BitmapFactory.decodeFile(uri);

    //  re-scale
    Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);

    // create file
    try {
      File newFile = BitmapUtility.saveBitmapToFile(newBitmap, options.outFormat, options.quality);
      promise.resolve(newFile.getAbsolutePath());
    } catch (FileNotFoundException e) {
      promise.reject("Error in ResizeStaticImage:", e);
    }


  }

  @ReactMethod
  public void ResizeStaticImagesBatch(ReadableArray uris, ReadableMap userOptions, Promise promise) {
    StaticOptions options = new StaticOptions(userOptions);

    try {
      // Create an array to store the paths of resized images
      WritableArray resizedPaths = Arguments.createArray();

      for (int i = 0; i < uris.size(); i++) {
        String uri = uris.getString(i);

        File file = new File(uri);
        if (!file.exists()) {
          // Add an error message to the array
          resizedPaths.pushString("Error: File does not exist for URI: " + uri);
          continue;
        }

        // create bitmap
        Bitmap bitmap = BitmapFactory.decodeFile(uri);

        // re-scale
        Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);

        // create file
        File newFile = BitmapUtility.saveBitmapToFile(newBitmap, options.outFormat, options.quality);
        resizedPaths.pushString(newFile.getAbsolutePath());
      }

      promise.resolve(resizedPaths);
    } catch (Exception e) {
      promise.reject("Error in ResizeStaticImagesBatch:", e);
    }
  }

}
